import { ComponentClass } from 'react';
import Taro, { Component } from '@tarojs/taro';
import { View } from '@tarojs/components';

import './Vcode.scss';
import { getVcode } from '../../api/wx';

type PageOwnProps = {
  text?: string;
  cellphone: string;
};
type PageState = {
  defaultText: string;
  counting: boolean;
  count: number;
  timer: number | null;
};

interface Vcode {
  props: PageOwnProps;
  state: PageState;
}

class Vcode extends Component {
  static externalClasses = ['vcode-classname'];

  constructor(props: PageOwnProps) {
    super(props);

    this.state = {
      defaultText: props.text ? props.text : '获取验证码',
      counting: false,
      count: 10,
      timer: null,
    };
  }

  componentWillUnmount() {
    const { timer } = this.state;
    if (timer) {
      clearTimeout(timer);
    }
  }

  clickHandle() {
    const { cellphone } = this.props;
    if (cellphone && cellphone.length == 11) {
      getVcode({
        tel: cellphone,
      })
        .then(res => {
          this.countStart();
          Taro.showToast({
            title: res.msg,
            mask: true,
          });
        })
        .catch(console.error);
    } else {
      Taro.showToast({
        title: '输入正确的手机号',
        icon: 'none',
        mask: true,
      });
    }
  }

  countStart() {
    this.setState({
      counting: true,
    });
    this.countDown();
  }

  countDown() {
    const { count } = this.state;
    if (count == 1) {
      this.setState({
        counting: false,
      });
    } else {
      const timer = setTimeout(() => {
        this.countDown();
      }, 1000);
      this.setState({
        count: count - 1,
        timer: timer,
      });
    }
  }

  render() {
    const { defaultText, counting, count } = this.state;
    return counting ? (
      <View className='vcode-classname Vcode Counting'>{`${count}S后重发`}</View>
    ) : (
      <View className='vcode-classname Vcode' onClick={this.clickHandle}>
        {defaultText}
      </View>
    );
  }
}

export default Vcode as ComponentClass<PageOwnProps, PageState>;
