import { ComponentClass } from 'react';
import Taro, { Component, Config } from '@tarojs/taro';
import { connect } from '@tarojs/redux';

import { View, Text, Input, Image, ScrollView } from '@tarojs/components';
import SearchIcon from '../../images/login/icon_search@2x.png';
import { Campus, CampusList } from '../../types/Campus/Campus';

import { fetchAllCampus } from '../../api/campus';
import { UserState, updateUserInfo } from '../../store/rootReducers/userinfo';
import User from 'src/types/User/User';

import './SelectCampus.scss';

type PageStateProps = {
  userinfo: UserState;
};
type PageDispatchProps = {
  updateUserInfo: (e: User) => void;
};

type PageState = {
  campusList: CampusList;
  filterList: CampusList;
  filterName: string;
  showToast: boolean;
  localText: string;
  timer: number | undefined;
};

type IProps = PageStateProps & PageDispatchProps;
interface SelectCampus {
  props: IProps;
  state: PageState;
}

@connect(
  ({ userinfo }) => ({
    userinfo,
  }),
  dispatch => ({
    updateUserInfo(entity: User) {
      dispatch(updateUserInfo(entity));
    },
  }),
)
class SelectCampus extends Component {
  config: Config = {
    navigationBarTitleText: '检索学校',
  };

  constructor(props: IProps) {
    super(props);
    this.state = {
      filterName: '',
      campusList: [],
      filterList: [],
      showToast: false,
      localText: '',
      timer: undefined,
    };
  }

  componentWillMount() {
    this.getList();
  }

  getList() {
    fetchAllCampus()
      .then(res => {
        console.log(res);
        let campusIndexArr: Array<string> = [];
        let campusList: CampusList = [];
        res.data.map((campus: Campus) => {
          let firstPin = campus.firstPin;
          if (firstPin) {
            let index = campusIndexArr.indexOf(firstPin);
            if (index > -1) {
              campusList[index].children.push(campus);
            } else {
              campusIndexArr.push(firstPin);
              campusList.push({
                firstPin: firstPin,
                children: [campus],
              });
            }
          }
        });
        this.setState({
          campusList: campusList,
          filterList: campusList,
        });
      })
      .catch(err => {
        console.log(err);
      });
  }

  searchList() {
    let { filterName, campusList } = this.state;
    if (filterName) {
      const filterList = campusList.reduce(
        (totalArr: CampusList, campusItem) => {
          let arr = campusItem.children.filter(
            i => i.name.indexOf(filterName) > -1,
          );
          if (arr.length > 0) {
            totalArr.push({
              firstPin: campusItem.firstPin,
              children: arr,
            });
          }
          return totalArr;
        },
        [],
      );
      this.setState({
        filterList: filterList,
      });
    } else {
      this.setState({
        filterList: campusList,
      });
    }
  }

  showToastHandle(text: string) {
    console.log(text);
    const { timer } = this.state;
    if (timer) {
      clearTimeout(timer);
    }
    const newTimer = setTimeout(() => {
      this.setState({
        showToast: false,
        localText: '',
      });
    }, 1000);
    this.setState({
      showToast: true,
      localText: text,
      timer: newTimer,
    });
  }

  bindCampus({ target: { dataset } }) {
    const {
      campus: { id, name, schoolId },
    } = dataset;

    this.props.updateUserInfo({
      campusId: id,
      campusName: name,
      schoolId: schoolId,
    });
    Taro.navigateBack();
  }

  render() {
    const {
      campusList,
      filterList,
      filterName,
      showToast,
      localText,
    } = this.state;
    return (
      <View className='Location topBr'>
        <View className='searchBox'>
          <Input
            className='br8 searchInput'
            value={filterName}
            onInput={({ detail: { value } }) =>
              this.setState({
                filterName: value.trim(),
              })
            }
            type='text'
            placeholder='请输入学校名称'
          />
          <View className='searchBtn' onClick={this.searchList}>
            <Image className='searchIcon' src={SearchIcon} />
            <Text>搜索</Text>
          </View>
        </View>
        <View className='infoBar'>
          {campusList.map(item => (
            <View
              key={item.firstPin}
              className='infoItem'
              onClick={() => this.showToastHandle(item.firstPin)}>
              {item.firstPin}
            </View>
          ))}
        </View>
        <View className={showToast ? 'show br8 infoToast' : 'br8 infoToast'}>
          {localText}
        </View>
        <ScrollView
          scroll-y
          className='locationList'
          scrollIntoView={localText}
          scrollWithAnimation>
          {filterList.map(campusItem => (
            <View key={campusItem.firstPin} className='locationArea'>
              <View id={campusItem.firstPin} className='locationTitle'>
                {campusItem.firstPin}
              </View>
              {campusItem.children.map(campus => (
                <View
                  key={campus.id}
                  className='locationItem'
                  data-campus={campus}
                  onClick={this.bindCampus}>
                  {campus.name}
                </View>
              ))}
            </View>
          ))}
        </ScrollView>
      </View>
    );
  }
}

export default SelectCampus as ComponentClass;
